package Family;

import Alignments.Alignment;
import Alignments.DuplicationAlignment;
import Exceptions.GeneralException;
import Likelihood.Calculator;
import Likelihood.Likelihood;
import Likelihood.StandardCalculator;
import Models.Model;
import Models.RateCategory;
import Optimizers.GoldenSection;
import Optimizers.Optimizer;
import Parameters.Parameters;
import Trees.Tree;
import java.io.File;
import java.util.HashMap;


public class Family
{

    public static void main(String[] args) throws GeneralException
    {
        //The maximum family size
        int num = 75;
        
        //Create the array to represent the matrix - one bigger than the
        //maximum size to represent zero
        String[][] matrix = new String[num+1][num+1];
        //Create the map from state (stroings "0", "1", "2", ..., "75") to position
        //in matrix
        HashMap<String,Integer> map = new HashMap<>();
        for (int i = 0; i <= num; i++)
        {
            map.put(Integer.toString(i), i);
        }
        //Set the rate for a change in size of one to 1.0 and all other rates
        //to zero
        for (int i = 0; i <= num; i++)
        {
            for (int j = 0; j <= num; j++)
            {
                if (Math.abs(i-j) == 1)
                {
                    matrix[i][j] = "1.0";
                }
                else
                {
                    matrix[i][j] = "0.0";
                }
            }
        }       
        
        //Create the model using the stationary frequencies for the root distribution
        Model m = new Model(new RateCategory(matrix,RateCategory.FrequencyType.STATIONARY,map));
        
        //Load the alignment
        Alignment a = DuplicationAlignment.fromFile(new File("src\\Family\\data\\families.dat"));
        //And the set of all missing sites - represented by another laignment
        Alignment missing = DuplicationAlignment.fromFile(new File("src\\Family\\data\\missing.dat"));
        
        //Load the tree
        Tree t = Tree.fromFile(new File("src\\Family\\data\\tree.dat"));
        
        //Create the calculator
        Calculator c = new StandardCalculator(m,a,t,missing);
        
        //Get the parameters - in this instance the model has none so it's just
        //those from the tree
        Parameters p = t.getParametersForEstimation();
        
        //Create the optimizer
        Optimizer o = new GoldenSection(GoldenSection.ProgressLevel.PARAMETER);
        
        //Optimize
        Likelihood l = o.maximise(c, p);
        
        //Print out the optimized parameters and the likelihood
        System.out.println(l.getParameters());
        System.out.println();
        System.out.println(l.getLikelihood());
    }
    
}
