package RY;

import Alignments.Alignment;
import Alignments.Ambiguous;
import Alignments.PhylipAlignment;
import Exceptions.GeneralException;
import Likelihood.Calculator;
import Likelihood.Likelihood;
import Likelihood.StandardCalculator;
import Likelihood.StandardLikelihood;
import Models.Model;
import Models.RYModelFactory;
import Optimizers.GoldenSection;
import Optimizers.Optimizer;
import Parameters.Parameters;
import Trees.Tree;
import java.io.File;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class RY
{
    public static void main(String[] args) throws GeneralException
    {
        //Read in the tree
        Tree t = Tree.fromFile(new File("src\\RY\\data\\tree.dat"));
        
        //Read in the DNA alignment
        Alignment a = PhylipAlignment.fromFile(new File("src\\RY\\data\\alignment.dat"));
        
        //Create a map of how to recode the characters to RY
        Map<String,String> recode = new HashMap<>();
        recode.put("C","Y");
        recode.put("T","Y");
        recode.put("A","R");
        recode.put("G","R");
        
        //Create a set of all RY chsracters
        Set<String> rany = new HashSet<>();
        rany.add("R");
        rany.add("Y");
        //And create a map mapping - and N to any character
        Map<String,Set<String>> ram = new HashMap<>();
        ram.put("-",rany);
        ram.put("N",rany);
        //Create the ambiguous object
        Ambiguous rambig = new Ambiguous(ram);
        
        //Recode the DNA alignment to RY passing information on ambiguous charcaters
        Alignment ra = a.recode(recode,rambig);
        
        //Get the parameters from the tree
        Parameters p = t.getParametersForEstimation();
        
        //Create the RY model, passing the parameters object which will
        //be updated with the parameters from the model
        Model ry = RYModelFactory.RY(p);
        
        //Create the calculator
        Calculator<StandardLikelihood> c = new StandardCalculator(ry,ra,t);
        
        //Create the optimizer
        Optimizer o = new GoldenSection();
        
        //Optimize
        Likelihood l = o.maximise(c, p);
        
        //Print out the optimized parameters and the likelihood
        System.out.println(l.getParameters());
        System.out.println();
        System.out.println(l.getLikelihood());
    }
    
}
